@@ -1,6 +1,6 @@
 import boto3
-from aws.region import get_ec2
-
+from aws.region import get_ec2, get_ssm
+import time
 
 def list_regions():
     """
@@ -143,7 +143,7 @@ def list_azs():
 
     """
 
-    return get_ec2().describe_availability_zones().get('AvailabilityZones')
+    return boto3.client('ec2').describe_availability_zones().get('AvailabilityZones')
 
 
 ###########################################################################
@@ -180,6 +180,10 @@ def set_current_vpcid(vpc: dict = None, vpcid: str = None):
 
 
 def get_current_vpcid():
+    """
+    Gets the ID of the current default VPC.
+    :return: The ID of the default VPC.
+    """
     global _default_vpcid
 
     if _default_vpcid is None:
@@ -233,7 +237,13 @@ def get_vpc(default: bool = False, name: str = None, vpcid: str = None):
     return None
 
 
-def get_vpcid(vpc: str):
+def get_vpcid(vpc: dict):
+    """
+    Gets the ID from a VPC dict.
+
+    :param vpc: A dict containing information about a VPC. Usually the output of get_vpc().
+    :return: The ID of the vpc
+    """
     return vpc.get('VpcId')
 
 
@@ -851,6 +861,11 @@ def create_key_pair(name: str):
 
 
 def list_key_pairs(name: str = None):
+    """
+    Lists the names of key pairs in the AWS account.
+    :param name: If provided, only return key pairs with this name. Should return a list with a single entry if it exists.
+    :return: a list of key pairs, either in the account or that match the name.
+    """
     kps = get_ec2().describe_key_pairs().get('KeyPairs')
     if name is not None:
         return [
@@ -863,23 +878,55 @@ def list_key_pairs(name: str = None):
 
 
 def key_pair_exists(name: str):
+    """
+    Finds if a key pair exists.
+
+    :param name: The name of the sought key pair
+    :return: True if there is a key pair with the provided name.
+    """
     kps = list_key_pairs(name)
     return len(kps) > 0
 
 
 def delete_key_pair(name: str):
+    """
+    Deletes a key pair record if it exists.
+
+    :param name: The name of the key pair to delete.
+    :return: None if the key pair does not exist. Returns metadata about the deletion if it does.
+    """
     if key_pair_exists(name):
         return get_ec2().delete_key_pair(KeyName=name)
     else:
         return None
 
+def wait_for_key_pair(name: str, interval: int = 10, timeout: int = 120):
+    """
+    Waits for a key pair to exist.
+
+    :param name: The name of the key pair we're waiting on.
+    :param interval: How often to check if the key pair exists (in seconds).
+    :param timeout: How long to wait before we time out (in seconds).
+    :return: True if the key pair exists and we don't timed out. False if we time out.
+    """
+    for i in range(timeout // interval + 1):
+            if key_pair_exists(name):
+                return True
+            if i <= timeout // interval:
+                print(f'waiting for key_pair {name}')
+                time.sleep(interval)
+    return False
 
 ###########################################################################
 # AMIs
 ###########################################################################
 
 def get_linux2_ami():
-    """Returns the AMI ID for the latest version of Linux 2 AMI"""
+    """
+    Gets the AMI for the latest HVM x64 version of Amazon Linux 2
+    
+    :return: The AMI for the lastest Amazon Linux 2 release.
+    """
     response = get_ec2().describe_images(
         Filters=[
             {
@@ -911,6 +958,12 @@ def get_linux2_ami():
 
 
 def get_image(imageid: str):
+    """
+    Gets details about an image.
+
+    :param imageid: The ID of the image you're querying for.
+    :return: Details about the image. See `boto3's docs <https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/ec2.html#EC2.Client.describe_images>`_ for more details.
+    """
     response = get_ec2().describe_images(
         ImageIds=[imageid]
     )['Images']
@@ -1289,7 +1342,7 @@ def get_tag_value(aws_obj: dict, key: str):
     return None
 
 
-def get_tags(aws_obj: dict):
+def get_ec2_tags(aws_obj: dict = None, name: str = None, instance_id: str = None):
     """
     Returns the tags in a standard dictionary format, simplified from the
     original AWS Tags format.
@@ -1312,6 +1365,12 @@ def get_tags(aws_obj: dict):
         { 'Name': 'default-vpc', 'EnvType': 'dev' }
 
     """
+
+    if name:
+        aws_obj = get_instance(name = name)
+    elif instance_id:
+        aws_obj = get_instance(instance_id = instance_id)
+
     tags = aws_obj.get('Tags')
 
     if tags is None:
@@ -1324,8 +1383,16 @@ def get_tags(aws_obj: dict):
 
 
 def match_tags(tags: dict, aws_obj: dict):
+    """
+    This function compares a collection of tags to those on an EC2 resource. If the object does not have
+    a tag present in the supplied collection or the two tags' values don't match, this returns false.
 
-    obj_tags = get_tags(aws_obj)
+    :param tags: A collection of key/value pairs to compare to a resource's tags.
+    :param aws_obj: the object with tags we are comparing to the first parameter's key/value pairs.
+    :return: if all keys in tags are present in aws_obj and have the same value as in tags.
+    """
+
+    obj_tags = get_ec2_tags(aws_obj)
 
     for key in tags.keys():
         if obj_tags.get(key) is None:
@@ -1336,3 +1403,95 @@ def match_tags(tags: dict, aws_obj: dict):
 
     return True
 
+def set_tag(key: str, value: str, name: str = None, instance_id: str = None):
+    """
+    Sets a tag for an EC2 instance to a given value.
+
+    :param key: The tag key to set
+    :param value: The new/updated tag's value
+    :param name: The instance's name. Required if parameter instance_id is not supplied.
+    :param instance_id: The instance's id. Required if parameter name is not supplied.
+    :return: True if the instance is successfully updated with a tag key = value.
+             None if both name and id are missing or if the corresponding instance doesn't exist.
+    """
+    if name is None and instance_id is None:
+        print('Must supply an instance name or id to tag')
+        return None
+    instance = get_instance(name = name, instance_id = instance_id)
+    if instance == None:
+        return None
+    tag = {
+      'Key': key,
+      'Value': value
+    }
+    get_ec2().create_tags(Resources = [get_instance_id(instance)], Tags = [tag])
+    return True
+    
+
+def wait_for_ready(name: str = None, instance_id: str = None, interval: int = 10, timeout: int = 120):
+    """
+    Waits for an instance to have a tag "Ready" with the value True. We set that tag at the end of each
+    instance's UserData/startup script, which lets us know that all software has finished installing.
+
+    :param name: The instance's name. Required if parameter instance_id is not supplied.
+    :param instance_id: The instance's id. Required if parameter name is not supplied.
+    :param interval: This function polls the instance every [interval] seconds. Defaults to 10.
+    :param timeout: After [timeout] seconds, the function returns False. Defaults to 120.
+    :return: True once the instance is ready. False if the instance is not ready by the time we hit timeout.
+             None if name and id are not supplied or if the corresponding instance doesn't exist.
+    """
+    if name is None and instance_id is None:
+       print('Must supply an instance name or id to wait on')
+       return None
+    instance = None
+    instance = get_instance(name = name, instance_id = instance_id)
+    if instance is None:
+        print(f'Instance not found: {name} / {instance_id}')
+        return None
+    for i in range(timeout // interval + 1):
+        ready = get_tag_value(instance, 'Ready')
+        if ready == 'True' or ready == 'true' or ready == True:
+            return True
+        if i <= timeout // interval:
+            print(f'waiting for {get_instance_id(instance)} to be ready')
+            time.sleep(interval)
+            instance = get_instance(instance_id = get_instance_id(instance))
+    return False
+
+
+###########################################################################
+# SSM
+###########################################################################
+
+def ssm_run_shell_script(name: str = None, instance_id: str = None, commands = [], directory: str = '/home/ec2-user/', comment: str = 'Ran a shell script'):
+    """
+    Runs a shell script on an instance via secure systems manager. If the instance does not
+    have the proper SSM permissions, the call will fail. Note that this does not wait for the
+    script to finish executing; if you want to do that, we recommend starting and ending the
+    script with API calls setting the 'Ready' tag to False and True respectively. That way,
+    you can follow up the call with the wait_for_ready() function.
+
+    :param name: The instance's name. Required if parameter instance_id is not supplied.
+    :param instance_id: The instance's id. Required if parameter name is not supplied.
+    :param commands: A list containing commands to execute. Each line of the sequence should be a string in the list.
+    :param directory: The directory to execute the script from. Defaults to ec2-user's home directory.
+    :param comment: A comment to attach to the command. Visible on the AWS web console in command history. Defaults to 'Ran a shell script'.
+    :return: The response from AWS to the command. See `the boto3 docs <https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/ssm.html#SSM.Client.send_command>`_ for spec.
+    """
+    if instance_id is not None:
+        iid = instance_id
+    elif name:
+        iid = get_instance_id(get_instance(name))
+    if iid is None:
+        print('Must specify an instance to run the script on')
+        return None
+    response = get_ssm().send_command(
+        InstanceIds = [iid],
+        DocumentName = 'AWS-RunShellScript',
+        Comment = comment,
+        Parameters = {
+          'commands': commands,
+          'workingDirectory': [directory]
+        }
+    )
+    return response
\ No newline at end of file
