@@ -1,15 +1,13 @@
 import boto3
 import ast
-from aws.region import get_elbv2
-from aws.region import get_elb
 from aws.ec2 import get_instance, set_current_vpcid, get_vpc
-
+from aws.region import get_elb, get_elbv2
 
 ###########################################################################
 # Elastic Load Balancers
 ###########################################################################
 
-def list_elbs():
+def list_classic_loadbalancer():
     """
     List all the load balancers.
 
@@ -155,11 +153,11 @@ def list_elbs():
         ]
 
     """
-    elbList = boto3.client('elb').describe_load_balancers().get('LoadBalancerDescriptions')
+    elbList = get_elb().describe_load_balancers().get('LoadBalancerDescriptions')
     return elbList
 
 
-def get_elb(name: str = None):
+def get_classic_loadbalancer(name: str = None):
     """
     List a specific load balancer. The load balancer's name must be provided for the search
 
@@ -234,7 +232,7 @@ def get_elb(name: str = None):
         ]
 
     """
-    elbList = list_elbs()
+    elbList = list_classic_loadbalancer()
     if name is not None:
         for elb in elbList:
             if elb.get('LoadBalancerName') == name:
@@ -248,7 +246,7 @@ def get_elb(name: str = None):
 # Elastic Load Balancers, Version 2
 ###########################################################################
 
-def list_elbv2s():
+def list_application_loadbalancer():
     """
     List all the application load balancers
 
@@ -388,11 +386,11 @@ def list_elbv2s():
         ]
 
     """
-    elbList = boto3.client('elbv2').describe_load_balancers().get('LoadBalancers')
+    elbList = get_elbv2().describe_load_balancers().get('LoadBalancers')
     return elbList
 
 
-def get_elbv2(arn: str = None, name: str = None):
+def get_application_loadbalancer(arn: str = None, name: str = None):
     """
     Get the a specific application load balancer. A name or arn must be specified for the search
 
@@ -450,7 +448,7 @@ def get_elbv2(arn: str = None, name: str = None):
 
     """
 
-    elbList = list_elbv2s()
+    elbList = list_application_loadbalancer()
     if name is not None:
         for elb in elbList:
             if elb.get('LoadBalancerName') == name:
@@ -540,16 +538,16 @@ def list_target_groups():
         ]
 
     """
-    targetGroupList = boto3.client('elbv2').describe_target_groups().get('TargetGroups')
+    targetGroupList = get_elbv2().describe_target_groups().get('TargetGroups')
     return targetGroupList
 
 
 def get_target_group(name: str = None, arn: str = None):
     """
-    :param name: The name of the Target Group
-    :param arn: The ARN of the Target Group
+    Gets a target group with a specified name or ARN
+    :param name: The name of the Target Group. Required if parameter arn isn't supplied.
+    :param arn: The ARN of the Target Group. Required if parameter name isn't supplied.
     :return: `TargetGroup Details <https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/elbv2.html#ElasticLoadBalancingv2.Client.describe_target_groups>`_
-
     """
     tgList = list_target_groups()
     if name is not None:
@@ -563,49 +561,83 @@ def get_target_group(name: str = None, arn: str = None):
     else:
         raise Exception('Either name or arn of the target group must be provided for the search')
 
-def add_target(vpc_name: str = None, port: int = 443, target_name: str = None, target_arn: str = None, group_name: str = None, group_arn: str = None):
-  if vpc_name is None:
-    print('must supply vpc name')
-    return False
-  if target_name is None and target_arn is None:
-    print('must supply target name or arn')
-    return False
-  if group_name is None and group_arn is None:
-    print('must supply group name or arn')
-    return False
-  print(f"setting vpc to {vpc_name}")
-  set_current_vpcid(vpc = get_vpc(name = vpc_name))
+def add_target(port: int = None, target_name: str = None, target_arn: str = None, group_name: str = None, group_arn: str = None):
+    """
+    Adds a target to a target group.
+    
+    :param port: The port to expose the target on. Defaults to the default port of the target group.
+    :param target_name: The name (tag, Key = Name) of the target instance. Required if parameter target_arn is not supplied.
+    :param target_arn: The ARN of the target instance. Required if parameter target_name is not supplied.
+    :param group_name: The name (tag, Key = Name) of the target group. Required if parameter group_arn is not supplied.
+    :param group_arn: The ARN of the target group. Required if parameter group_name is not supplied.
+    :return: If the target is successfully added to the target group.
+    """
+    if target_name is None and target_arn is None:
+        print('must supply target name or arn')
+        return False
+    if group_name is None and group_arn is None:
+        print('must supply group name or arn')
+        return False
 
-  # make sure the instance exists
-  if target_name is not None:
-    target = get_instance(target_name)
-    if target is None:
-      print(f"instance {target_name} not found")
-      return False
-  else: # arn
-    target = get_instance(instance_id = target_arn)
-    print(f"instance {target_arn} not found")
-  # make sure the target group exists
-  if group_name is not None:
-    group = get_target_group(group_name)
-    if group is None:
-      print(f"group {group_name} not found")
-      return False
-  else: # arn
-    group = get_target_group(arn = group_arn)
-    if group is None:
-      print(f"group {group_arn} not found")
-      return False
-  # add the retrieved target to the target group
-  t_param = {
-    'Id': target['InstanceId'],
-    'Port': port
-  }
-  boto3.client('elbv2').register_targets(TargetGroupArn = group['TargetGroupArn'], Targets = [t_param])
+    # make sure the instance exists
+    if target_name is not None:
+        target = get_instance(target_name)
+        if target is None:
+            print(f"instance {target_name} not found")
+            return False
+    else: # arn
+        target = get_instance(instance_id = target_arn)
+        print(f"instance {target_arn} not found")
+    # make sure the target group exists
+    if group_name is not None:
+        group = get_target_group(group_name)
+        if group is None:
+            print(f"group {group_name} not found")
+            return False
+    else: # arn
+        group = get_target_group(arn = group_arn)
+        if group is None:
+            print(f"group {group_arn} not found")
+            return False
+    # add the retrieved target to the target group
+    if port is None:
+        port = group['Port']
+    t_param = {
+        'Id': target['InstanceId'],
+        'Port': port
+    }
+    get_elbv2().register_targets(TargetGroupArn = group['TargetGroupArn'], Targets = [t_param])
 
-  return True
+    return True
 
-def get_tags(arn: str) -> list:
+def get_targets_in_group(group_name: str = None, group_arn: str = None):
+    """
+    Gets a list of targets in a group.
+
+    :param group_name: The name of the target group. Required if parameter group_arn isn't supplied.
+    :param group_arn: The ARN of the target group. Required if parameter group_name isn't supplied.
+    :return: A list of targets in the group.
+    """
+
+    if group_name is not None:
+        group = get_target_group(group_name)
+        if group is None:
+            print(f"group {group_name} not found")
+            return False
+        group_arn = group['TargetGroupArn']
+    elif group_arn is not None:
+        group = get_target_group(arn = group_arn)
+        if group is None:
+            print(f"group {group_arn} not found")
+            return False
+    else:
+        print('must supply group name or arn')
+        return False
+
+    return get_elbv2().describe_target_health(TargetGroupArn = group_arn)['TargetHealthDescriptions']
+    
+
+def get_elb_tags(arn: str) -> list:
     """
     Get list of tags associated with the LoadBalancer or TargetGroup
 
@@ -621,9 +653,7 @@ def get_tags(arn: str) -> list:
 
     """
 
-
-
-    tagList = boto3.client('elbv2').describe_tags(ResourceArns=[arn]).get('TagDescriptions')[0].get('Tags')
+    tagList = get_elbv2().describe_tags(ResourceArns=[arn]).get('TagDescriptions')[0].get('Tags')
     res = {}
     for x in tagList:
         key = x.get('Key')
